//
//  BFMusicSearchController.swift
//  BFRecordScreenKit
//
//  Created by Harry on 2022/3/3.
//

import Foundation
import BFUIKit
import BFMediaKit
import BFCommonKit
import BFNetRequestKit
import Alamofire
import MJRefresh

class BFMusicSearchController: BFBaseViewController {
    
    var choseAction : ((PQVoiceModel?) -> Void)?
    
    var cutActionCallback: ((PQVoiceModel) -> Void)?
    
    var loadedTimeRangesObserver : NSKeyValueObservation?

    var chosedIndexPath : IndexPath?

    var chosedCellStatu : BFMuicInfoCellState = .pause
    
    var searchResults = [PQVoiceModel]()
    
    var currPage = 1
    
    var musicVolume = 0 {
        didSet{
            slidV.value = Float(musicVolume) / 100.0
            uiForChangeVolume()
        }
    }
    var keywork : String?

    // 试听音乐
    let playerManager = BFMusicManager()
    
    lazy var searchTF : UITextField = {
        let searchTF = UITextField()
        let leftv = UIView(frame: CGRect(x: 0, y: 0, width: 30, height: 20))
        let iv = UIImageView(frame: CGRect(x: 6, y: 0, width: 20, height: 20))
        iv.contentMode = .scaleAspectFit
        iv.image = imageInRecordScreenKit(by: "search")
        leftv.addSubview(iv)
        searchTF.leftView = leftv
        searchTF.leftViewMode = .always
        searchTF.clearButtonMode = .never
       
        let redPlaceholderText = NSAttributedString(string: "歌名/歌手名",
                                                    attributes: [NSAttributedString.Key.foregroundColor:  UIColor.hexColor(hexadecimal: "#BDBDBD"),NSAttributedString.Key.font:UIFont.systemFont(ofSize: 14,weight: .regular)])
        searchTF.attributedPlaceholder = redPlaceholderText
        searchTF.font = UIFont.systemFont(ofSize: 14,weight: .regular)
        searchTF.delegate = self
        searchTF.returnKeyType = .search
        searchTF.textColor = UIColor.white
        searchTF.backgroundColor = UIColor.hexColor(hexadecimal: "#1d1d1d")
        searchTF.layer.cornerRadius = 18
        
        return searchTF
    }()
    
    lazy var progressL : UILabel = {
        let l = UILabel()
        l.textColor = UIColor.hexColor(hexadecimal: "#9d9d9d")
        l.font = UIFont.systemFont(ofSize: 13, weight: .regular)
        l.textAlignment = .center
        l.text = "0%"
        return l
    }()
    
    lazy var slidV : UISlider = {
        let v = UISlider()
        v.addTarget(self, action: #selector(valuChange(slid:)), for: .valueChanged)
        return v
    }()
    
    lazy var musicTb : UITableView = {
        let tb = UITableView(frame: .zero)
        tb.delegate = self
        tb.dataSource = self
        tb.separatorColor = UIColor.hexColor(hexadecimal: "#272727")
        tb.register(BFMusicInfoSearchCell.self, forCellReuseIdentifier: "BFMusicInfoSearchCell")
        tb.tableFooterView = UIView()
        tb.backgroundColor = .clear
//        tb.mj_footer = MJRefreshAutoFooter.init(refreshingBlock: {[weak self] in
//            guard let wself = self else { return }
//            if wself.searchResults.count > 0 {
//                wself.requestSearchMusic()
//            }
//        })
        return tb
    }()
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        searchTF.becomeFirstResponder()
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        uiForChangeVolume()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        let closeBtn = UIButton()
        closeBtn.setImage(imageInRecordScreenKit(by: "search_close"), for: .normal)
        closeBtn.frame = CGRect(x: 12, y: statusBarHeight + 2, width: 40, height: 40)
        closeBtn.addTarget(self, action: #selector(closeAction), for: .touchUpInside)
        view.addSubview(closeBtn)
        
        searchTF.frame = CGRect(x: 62, y: statusBarHeight + 4, width: cScreenWidth - 62 - 18, height: 36)
        view.addSubview(searchTF)

        view.addSubview(musicTb)
        musicTb.snp.makeConstraints { make in
            make.left.right.equalToSuperview()
            make.top.equalTo(statusBarHeight + 44)
            make.bottom.equalTo(-100)
        }
        
        let soundIV = UIImageView(image: imageInRecordScreenKit(by: "soundBtn"))
        soundIV.contentMode = .scaleAspectFit
        view.addSubview(soundIV)
        soundIV.snp.makeConstraints { make in
            make.left.equalTo(18)
            make.top.equalTo(musicTb.snp.bottom).offset(36)
            make.width.height.equalTo(28)
        }
        
        view.addSubview(slidV)
        slidV.frame = CGRect(x: 56, y: cScreenHeigth - 59, width: cScreenWidth - 56 - 18, height: 18)

        progressL.frame = CGRect(x: slidV.x - 7, y: slidV.y - 24, width: 36, height: 16)
        view.addSubview(progressL)
        
//        musicVolume = 0.2
        
        playerManager.playStatusCallback = {[weak self] playerItem, playStatus in
            guard let wself = self else { return }
            
            if let indx = wself.chosedIndexPath, let cell = wself.musicTb.cellForRow(at: indx) as? BFMuicInfoCell, (playerItem?.asset as? AVURLAsset)?.url.absoluteString ?? "b" == cell.data?.musicPath ?? "a" {
                DispatchQueue.main.async {
                    cell.status = playStatus                    
                }
            }
            wself.chosedCellStatu = playStatus
        }
        
    }
    
    func choseCell() -> BFMuicInfoCell? {
        if let indpx = chosedIndexPath, let cell = musicTb.cellForRow(at: indpx) as? BFMuicInfoCell {
            return cell
        }
        return nil
    }
    
    @objc func closeAction(){
        self.dismiss(animated: true, completion: nil)
    }
    
    @objc func valuChange(slid:UISlider) {
        musicVolume = Int(slid.value * 100)
    }
    
    func uiForChangeVolume() {

        progressL.text = String(format: "%d%%", musicVolume)
        
        var frame = progressL.frame
        frame.origin.x = slidV.x - 7 + ((slidV.width - 28) / 100.0) * CGFloat(musicVolume)
        progressL.frame = frame
        
        playerManager.player.volume = Float(musicVolume) / 100.0

    }
}

extension BFMusicSearchController : UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return searchResults.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "BFMusicInfoSearchCell")
        
        if let cell = cell as? BFMusicInfoSearchCell{
            cell.data = searchResults[indexPath.row]

            cell.useCallback = {[weak self, weak cell] in
                guard let wself = self else { return }
                wself.playerManager.pause()
                cell?.status = .pause
                cell?.data?.volume = (Int)(wself.slidV.value * 100.0)
                wself.choseAction?(cell?.data)
                wself.dismiss(animated: true, completion: nil)
            }
            cell.cutCallBack = {[weak self, weak cell] in
                guard let wself = self else { return }
                wself.playerManager.pause()
                cell?.status = .pause
                cell?.data?.volume = (Int)(wself.slidV.value * 100.0)

                if let data = cell?.data {
                    wself.dismiss(animated: true) {[weak self] in
                        guard let wself = self else { return }
                        wself.cutActionCallback?(data)
                    }
                }
            }

            if cell.data?.isSelected ?? false {
                chosedIndexPath = indexPath
                cell.status = chosedCellStatu
            }else{
                cell.status = .normal
            }
        }
        
        cell?.selectionStyle = .none
        
        return cell!
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 64
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        
        if let cell = tableView.cellForRow(at: indexPath) as? BFMusicInfoSearchCell {
            if chosedIndexPath == indexPath {
                if cell.status != .pause{
                    cell.status = .pause
                    playerManager.pause()
                }else{
                    cell.status = .loading
                    playerManager.play()
                }
                return
            }
            cell.changeSelected(true)
            
            chosedIndexPath = indexPath
            
            if cell.status == .normal {
                if let urlStr = cell.data?.musicPath, let url = URL(string: urlStr){
                    BFLog(1, message: "歌曲地址: \(url)")
                    if url.absoluteString != (playerManager.currPlayUrlPath ?? "b") {
                        playerManager.replaceCurrentItem(musicPath: urlStr)
                        cell.status = .loading
                    }else {
                        cell.status = .playing
                    }
                    playerManager.play()
                }
            }else if cell.status == .pause{
                playerManager.play()
            } else{
                playerManager.pause()
                cell.status = .pause
            }
            chosedCellStatu = cell.status

        }
    }
    
    func tableView(_ tableView: UITableView, didDeselectRowAt indexPath: IndexPath) {
        searchResults[indexPath.row].isSelected = false
        let cell = tableView.cellForRow(at: indexPath) as? BFMusicInfoSearchCell
        cell?.status = .normal
        cell?.changeSelected(false)
        
        chosedIndexPath = nil
        playerManager.pause()
    }
    
    // MARK: - 搜素接口
    func requestSearchMusic() {
        if keywork?.count ?? 0 < 1 {
            cShowHUB(superView: nil, msg: "搜索不能为空")
            return
        }

        BFNetRequestAdaptor.postRequestData(url: PQENVUtil.shared.materialSearchApi + searchBGMMaterialUrl, parames: ["keyWord": keywork, "pageNo": 1, "pageSize": 200], commonParams: commonParams(), encoding: JSONEncoding.default, isJsonEncodingNormal: true) { [weak self] response, _, error, _ in
            DispatchQueue.global().async {[weak self] in
                guard let wself = self else { return }
                
                var bgmList = [PQVoiceModel]()
                if response is NSNull || response == nil {
                    DispatchQueue.main.async {
                        cShowHUB(superView: nil, msg: "网络连接失败，请检查网络后重试")
                    }
                    return
                }
                let tempArr = (response as? [String: Any])?["entityList"] as? [[String: Any]]
                if tempArr != nil {
                    for item in tempArr! {
                        let tempModel = PQVoiceModel(jsonDict: item)
                        tempModel.volume = 30
                        tempModel.voiceType = VOICETYPT.BGM.rawValue
                        bgmList.append(tempModel)
                    }
                }
                if wself.currPage == 1 {
                    wself.searchResults.removeAll()
                }
                wself.searchResults.append(contentsOf: bgmList)
                wself.currPage += 1
                DispatchQueue.main.async {[weak self] in
                    guard let wself = self else { return }
                    
                    if wself.searchResults.count > 0 {
//                        wself.musicTb.tableFooterView = UIView()
                    }else{
                        wself.musicTb.tableFooterView = wself.createNoDataView()
                    }
                    wself.musicTb.reloadData()
                }
            }
        }
    }
    
    func createNoDataView() -> UIView {
        let back = UIView(frame: CGRect(x: 0, y: 0, width: cScreenWidth, height: 200))
        
        let l = UILabel(frame: CGRect(x: 40, y: 60, width: cScreenWidth - 80, height: 80))
        l.text = "没有搜索到相关音乐\n换个词再试试吧"
        l.textColor = UIColor.hexColor(hexadecimal: "#787878")
        l.textAlignment = .center
        l.numberOfLines = 2
        l.font = UIFont.systemFont(ofSize: 17)
        back.addSubview(l)
        
        return back
    }
    
    func resetRearch() {
        playerManager.pause()
        chosedCellStatu = .normal
        chosedIndexPath = nil
        currPage = 1
    }
}

extension BFMusicSearchController : UITextFieldDelegate {
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        if let keyword = textField.text, keyword.replacingOccurrences(of: " ", with: "", options: .literal, range: nil).count > 0 {
            resetRearch()
            keywork = keyword
            requestSearchMusic()
        }else{
            cShowHUB(superView: nil, msg: "搜索不能为空")
        }
        
        textField.resignFirstResponder()
        
        
        return true
    }
    
    
}
