//
//  BFMuicInfoCell.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/3/1.
//

import Foundation
import BFMediaKit
import BFCommonKit
import BFMaterialKit

enum BFMuicInfoCellState {
    case normal, playing, pause, loading
}

class BFMuicInfoCell: UITableViewCell {
    var cutCallBack : (() -> Void)?
    var useCallback : (() -> Void)?

    var data:PQVoiceModel?{
        didSet{
            addData()
        }
    }
    
    var status : BFMuicInfoCellState = .normal {
        didSet {
            switch status {
            case .normal:
                maskIV.image = nil
                audioStateView.isHidden = true

            case .playing:
                maskIV.image = imageInRecordScreenKit(by: "markPlaying")
                stopLoading()
                audioStateView.isHidden = false
                audioStateView.kf.setImage(with: URL(fileURLWithPath: currentBundle()!.path(forResource: "stuckPoint_music_playing", ofType: ".gif")!))
                
            case .pause:
                maskIV.image = imageInRecordScreenKit(by: "markSeleced")
                stopLoading()
                audioStateView.isHidden = false
                audioStateView.image = imageInRecordScreenKit(by: "stuckPoint_music_pause")

            case .loading:
                maskIV.image = imageInRecordScreenKit(by: "markSeleced")
                audioStateView.image = imageInRecordScreenKit(by: "audioLoding")?.withRenderingMode(.alwaysTemplate)
                loadingAnimation()
                audioStateView.isHidden = false
            }
        }
    }
    
    lazy var iconIV : UIImageView = {
        let iv = UIImageView()
        iv.layer.cornerRadius = 21
        iv.layer.masksToBounds = true
        
        return iv
    }()
    
    lazy var maskIV : UIImageView = {
        let iv = UIImageView()
        iv.layer.masksToBounds = true
        iv.layer.cornerRadius = 21
        iv.layer.masksToBounds = true
        iv.backgroundColor = UIColor.init(white: 0, alpha: 0.5)
        
        return iv
    }()

    // 语音播放动画和暂停播放
    lazy var audioStateView: UIImageView = {
        let iv = UIImageView()
        iv.image = imageInRecordScreenKit(by: "audioLoding")
        iv.contentMode = .scaleAspectFit
        iv.isHidden = true
        return iv
    }()
    
    lazy var titleL : UILabel = {
        let l = UILabel()
        l.text = "歌曲"
        l.textColor = .white
        l.font = UIFont.systemFont(ofSize: 16, weight: .regular)
        l.textAlignment = .left
        return l
    }()
    
    lazy var cutBtn : UIButton = {
        let cutBtn = UIButton()
        cutBtn.tag = 2001
        cutBtn.setImage(imageInRecordScreenKit(by: "clip"), for: .normal)
        cutBtn.addTarget(self, action: #selector(cutAction), for: .touchUpInside)
        return cutBtn
    }()
    
    lazy var useBtn : UIButton = {
        let useBtn = UIButton()
        useBtn.tag = 2002
        useBtn.setTitle("使用", for: .normal)
        useBtn.backgroundColor = UIColor.hexColor(hexadecimal: "#389AFF")
        useBtn.layer.cornerRadius = 4
        useBtn.titleLabel?.font = UIFont.systemFont(ofSize: 14)
        useBtn.addTarget(self, action: #selector(useBtnAction), for: .touchUpInside)
        return useBtn
    }()
    
    override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
        super.init(style: style, reuseIdentifier: reuseIdentifier)
        self.backgroundColor = .clear
        contentView.backgroundColor = .clear
        
        iconIV.frame = CGRect(x: 18, y: 11, width: 42, height: 42)
        maskIV.frame = iconIV.bounds
        audioStateView.frame = CGRect(x: 8, y: 8, width: 26, height: 26)
        
        iconIV.addSubview(maskIV)
        iconIV.addSubview(audioStateView)
        contentView.addSubview(iconIV)
        

        titleL.frame = CGRect(x: iconIV.rightX + 10, y: 12, width: width - iconIV.rightX - 10 - 90, height: 40)
        contentView.addSubview(useBtn)
        useBtn.snp.makeConstraints { make in
            make.right.equalTo(-16)
            make.width.equalTo(0)
            make.height.equalTo(40)
            make.top.equalTo(12)
        }
                
        contentView.addSubview(cutBtn)
        cutBtn.snp.makeConstraints { make in
            make.right.equalTo(useBtn.snp.left).offset(-2)
            make.width.equalTo(0)
            make.height.equalTo(40)
            make.top.equalTo(12)
        }
        
        contentView.addSubview(titleL)
        titleL.snp.makeConstraints { make in
            make.left.equalTo(iconIV.snp.right).offset(10)
            make.top.equalTo(12)
            make.height.equalTo(40)
            make.right.equalTo(cutBtn.snp.left).offset(-2)
        }
        

    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func addData() {
        if let data = data{
            let plImg = imageInRecordScreenKit(by: "music_icon") ?? UIImage()
            iconIV.setNetImage(url: data.avatarUrl, placeholder: plImg)
            titleL.text = data.musicName
            
            changeSelected(data.isSelected)
            
        
        }
    }
    
    func changeSelected(_ selected: Bool){
        data?.isSelected = selected

        if selected{
            titleL.textColor = UIColor.hexColor(hexadecimal: "#389AFF")
            titleL.font = UIFont.systemFont(ofSize: 16, weight: .medium)
            showFunctionBtn()
        }else {
            titleL.textColor = UIColor.white
            titleL.font = UIFont.systemFont(ofSize: 16, weight: .regular)
            hiddenFunctionBtn()
        }
    }

    
    func showFunctionBtn() {
        cutBtn.snp.updateConstraints { make in
            make.width.equalTo(40)
        }
    }
    
    func hiddenFunctionBtn() {
        cutBtn.snp.updateConstraints { make in
            make.width.equalTo(0)
        }
    }
    
    @objc func cutAction() {
        cutCallBack?()
    }
    
    
    @objc func useBtnAction() {
        useCallback?()
    }
    
    func loadingAnimation(){
        let rotationAnim = CABasicAnimation(keyPath: "transform.rotation.z")
        // 2.设置动画的属性
        rotationAnim.fromValue = 0
        rotationAnim.toValue = Double.pi * 2
        rotationAnim.repeatCount = MAXFLOAT
        rotationAnim.duration = 1
        // 这个属性很重要 如果不设置当页面运行到后台再次进入该页面的时候 动画会停止
        rotationAnim.isRemovedOnCompletion = false
        // 3.将动画添加到layer中
        audioStateView.layer.add(rotationAnim, forKey: nil)

    }
    
    func stopLoading(){
        audioStateView.layer.removeAllAnimations()
    }
    
    deinit {
        stopLoading()
    }
}
